/**
 * 判断应用升级模块，从url地址下载升级描述文件到本地local路径
 * yanyilin@dcloud.io
 * 
 * 升级文件为JSON格式数据，如下：
{
	"appid":"拍拍Sale",
    "iOS":{
    	"version":"iOS新版本号，如：1.0.0",
    	"note":"iOS新版本描述信息，多行使用\n分割",
    	"url":"Appstore路径，如：itms-apps://itunes.apple.com/cn/app/hello-h5+/id682211190?l=zh&mt=8"
    },
    "Android":{
    	"version":"1.0.1",
    	"note":"拍拍Sale1.0.1",
    	"url":"http://a.app.qq.com/o/simple.jsp?pkgname=com.zhongde.guifei"
    }
}
 *
 */
(function(w){
	var server="http://admin.1314zhilv.com/resources/app/update/update.json",//获取升级描述文件服务器地址http://114.215.70.176:8028/resources/app/update/update.json
	localDir="update",localFile="update.json",//本地保存升级描述目录和文件名
	keyUpdate="updateCheck",//取消升级键名
	keyAbort="updateAbort",//忽略版本键名
	//checkInterval=604800000,//升级检查间隔，单位为ms,7天为7*24*60*60*1000=604800000, 如果每次启动需要检查设置值为0
	dir=null;
	/**
	 * 准备升级操作
	 * 创建升级文件保存目录
	 */
	function plusReady(){
		//defaultWaiting()
		function defaultWaiting(){//等待效果-------
			if(plus.os.name=='iOS'){
				plus.nativeUI.showWaiting( "加载中...",{height:"19%",width:"34%",padding:"6%",size:"17px"});//等待框样式
			}else{
				plus.nativeUI.showWaiting( "加载中...",{height:"20%",width:"34%",padding:"4%",size:"17px",loading:{height:"38px"}});//等待框样式
			}
		}
		// 打开doc根目录
		plus.io.requestFileSystem( plus.io.PRIVATE_DOC, function(fs){
			fs.root.getDirectory( localDir, {create:true}, function(entry){
				dir = entry;
				checkUpdate();
			}, function(e){
				console.log( "准备升级操作，打开update目录失败："+e.message );
				//sendLogger(e);
			});
		},function(e){
			console.log( "准备升级操作，打开doc目录失败："+e.message );
			//sendLogger(e);
		});
	}
	
	/**
	 * 检测程序升级
	 */
	function checkUpdate() {
		// 读取本地升级文件：本地文件，但是无论有没有本地文件都不创建文件
		dir.getFile( localFile, {create:false}, function(fentry){
			fentry.file( function(file){
				var reader = new plus.io.FileReader();
				//读取线上文件
				reader.onloadend = function ( e ) {
					var data = null;
					try{
						data=JSON.parse(e.target.result);
					}catch(e){
						console.log( "读取本地升级文件，数据格式错误！" );
						//sendLogger(e);
						return;
					}
					//拿本次数据去对比
					checkUpdateData( data );
				}
				//alert("本地："+JSON.stringify(file));
				//读取本地文件
				reader.readAsText(file);
				
			}, function(e){//读取本地升级文件失败
				console.log( "读取本地升级文件失败："+e.message );
				fentry.remove();
				//sendLogger(e);
			} );
		}, function(e){
			// 失败,表示文件不存在，需要从服务器获取升级数据
			getUpdateData();
			console.log( "本地升级文件不存在："+e.message);
			//sendLogger(e);
		});
	}
   
	/**
	 * 检查升级数据
	 */
	function checkUpdateData(j){
		//当前版本号：取自应用，并非update里面记录的版本
		var curVer=plus.runtime.version,
		inf = j[plus.os.name];
		//alert("inf:"+JSON.stringify(inf));
		if ( inf ){
			var srvVer = inf.version;
			// 判断是否存在忽略版本号
			var vabort = plus.storage.getItem( keyAbort );
			// 判断是否需要升级
			if ( compareVersion(curVer,srvVer) ) {
				plus.nativeUI.closeWaiting();//关闭等待框
				// 提示用户是否升级
				plus.nativeUI.confirm(inf.note, function(i) {
					if(1 == i.index) {
						//alert("立即更新");
						plus.runtime.openURL(inf.url);
						//window.location.href="anction-guide.html"//去引导页
					} else {
						//alert("取消");
						plus.storage.setItem(keyUpdate, (new Date()).getTime().toString());
						//window.location.href="anction-guide.html"//去引导页
					}
				}, inf.title, ["取　　消", "立即更新"]);
				//弹窗出来之后的逻辑
				//alert("立即更新，取　　消，之后");
			}else{
				//window.location.href="anction-guide.html"//去引导页
			}
		}
	}
	
	/**
	 * 从服务器获取升级数据
	 */
	function getUpdateData(){
		var xhr = new plus.net.XMLHttpRequest();
		xhr.onreadystatechange = function () {
	        switch ( xhr.readyState ) {
	        	 case 0:
	            	console.log( "xhr请求已初始化" );
	            break;
	            case 1:
	            	console.log( "xhr请求已打开" );
	            break;
	            case 2:
	            	console.log( "xhr请求已发送" );
	            break;
	            case 3:
	                console.log( "xhr请求已响应");
	                break;
	            case 4:
	                if ( xhr.status == 200 ) {
	                	// 保存到本地文件中
	                	dir.getFile( localFile, {create:true}, function(fentry){
	                		fentry.createWriter( function(writer){
	                			writer.onerror = function(){
	                				console.log( "获取升级数据，保存文件失败！" );
	                			}
	                			writer.write( xhr.responseText );
	                			//window.location.href="anction-guide.html"//去引导页
	                		}, function(e){
	                			console.log( "获取升级数据，创建写文件对象失败："+e.message );
	                			//sendLogger(e);//记录异常,并传给后台
	                		} );
	                	}, function(e){
	                		console.log( "获取升级数据，打开保存文件失败："+e.message );
	                		//sendLogger(e);
	                	});
	                } else{
	                	console.log( "获取升级数据，联网请求失败或者服务器错误："+xhr.status );
						//window.location.href="anction-guide.html"
	                }
	                break;
	            default :
	                break;
	        }
		}
		xhr.open( "GET", server );
		xhr.send();
	}
	
	/**
	 * 比较版本大小，如果新版本nv大于旧版本ov则返回true，否则返回false
	 * @param {String} ov
	 * @param {String} nv
	 * @return {Boolean} 
	 */
	function compareVersion( ov, nv ){
		if ( !ov || !nv || ov=="" || nv=="" ){
			return false;
		}
		var b=false,
		ova = ov.split(".",4),
		nva = nv.split(".",4);
		for ( var i=0; i<ova.length&&i<nva.length; i++ ) {
			var so=ova[i],no=parseInt(so),sn=nva[i],nn=parseInt(sn);
			if ( nn>no || sn.length>so.length  ) {
				return true;
			} else if ( nn<no ) {
				return false;
			}
		}
		if ( nva.length>ova.length && 0==nv.indexOf(ov) ) {
			return true;
		}
	}
	if ( w.plus ) {
		plusReady();
	} else {
		document.addEventListener("plusready", plusReady, false );
	}
})(window);

